@@ -689,7 +689,7 @@ nsMessengerOSXIntegration::BadgeDockIcon()
 }
 
 nsresult
-nsMessengerOSXIntegration::BadgeDockIconForTiger(nsAutoString& badgeString)
+nsMessengerOSXIntegration::BadgeDockIconForTiger(nsString& badgeString)
 {
   // On 10.4 we have to draw this manually, clearing any existing badge artifacts first.
   RestoreDockIcon();
