@@ -303,7 +303,11 @@ nsHttpHandler::Init()
     rv = InitConnectionMgr();
     if (NS_FAILED(rv)) return rv;
 
+#ifdef ANDROID
+    mProductSub.AssignLiteral(MOZ_APP_UA_VERSION);
+#else
     mProductSub.AssignLiteral(MOZ_UA_BUILDID);
+#endif
     if (mProductSub.IsEmpty() && appInfo)
         appInfo->GetPlatformBuildID(mProductSub);
     if (mProductSub.Length() > 8)
@@ -617,6 +621,7 @@ nsHttpHandler::BuildUserAgent()
                            mAppName.Length() +
                            mAppVersion.Length() +
                            mCompatFirefox.Length() +
+                           mCompatDevice.Length() +
 #ifdef FX104_OPTIMIZED_FOR
                            // needs to be length of SP "TenFourFox/Debugging"
                            // this is the longest string we support
@@ -636,8 +641,15 @@ nsHttpHandler::BuildUserAgent()
     mUserAgent += mPlatform;
     mUserAgent.AppendLiteral("; ");
 #endif
+#ifdef ANDROID
+    if (!mCompatDevice.IsEmpty()) {
+        mUserAgent += mCompatDevice;
+        mUserAgent.AppendLiteral("; ");
+    }
+#else
     mUserAgent += mOscpu;
     mUserAgent.AppendLiteral("; ");
+#endif
     mUserAgent += mMisc;
     mUserAgent += ')';
 
@@ -674,8 +686,7 @@ typedef BOOL (WINAPI *IsWow64ProcessP) (HANDLE, PBOOL);
 void
 nsHttpHandler::InitUserAgentComponents()
 {
-
-      // Gather platform.
+    // Gather platform.
     mPlatform.AssignLiteral(
 #if defined(ANDROID)
     "Android"
@@ -694,6 +705,18 @@ nsHttpHandler::InitUserAgentComponents()
 #endif
     );
 
+#if defined(ANDROID)
+    nsCOMPtr<nsIPropertyBag2> infoService = do_GetService("@mozilla.org/system-info;1");
+    NS_ASSERTION(infoService, "Could not find a system info service");
+
+    bool isTablet;
+    infoService->GetPropertyAsBool(NS_LITERAL_STRING("tablet"), &isTablet);
+    if (isTablet)
+        mCompatDevice.AssignLiteral("Tablet");
+    else
+        mCompatDevice.AssignLiteral("Mobile");
+#endif
+
     // Gather OS/CPU.
 #if defined(XP_OS2)
     ULONG os2ver = 0;
